/*
 * Copyright 2008 Dan Furie and Scott Linderman
 * 
 * This handles the state machine for the menu. The menu will allow users to set up
 * their shot by selecting a number of different settings. These settings include
 * the type of sensors to use for activation, the variables associated with the
 * chosen sensor, and other photo settings which are independent of the chosen sensor.
 * 
 * The menu will be designed as follows:
 * 
 * MAIN MENU
 * | -- SETUP
 * | -- | -- GENERAL SETTINGS (# pics, flash, etc)
 * | -- | -- SHOT TYPE
 * | -- | -- | -- Extended Exposure (time)
 * | -- | -- | -- Audio Trigger (threshold)
 * | -- | -- | -- Accelerometer (threshold)
 * | -- | -- | -- Photo Trigger (threshold)
 * | -- Start
 * | -- | -- | -- Take Extended Exposure
 * | -- | -- | -- Take Audio Triggered
 * | -- | -- | -- Take Accelerometer Triggered
 * | -- | -- | -- Take Photo Triggered
 * | -- About
 */

//include libraries for avr-gcc
#define F_CPU 16000000UL  // 16 MHz
#include <stdio.h>
#include <avr/interrupt.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <avr/eeprom.h>

// include libraries for project
#include "lcdLib.h"
#include "drawing.h"
#include "camera.h"

//start with test values for the buttons
#define SELECT 0x0e
#define PREV 0x0f
#define LEFT 0x0c
#define RIGHT 0x0d
#define UP 0x0a
#define DOWN 0x0b
#define NONE 0x10

//define menu state machine vals
#define MAIN_MENU 0
#define SETUP 1
#define START 2
#define ABOUT 3
#define GEN_SETTINGS 4
#define SHOT_TYPE 5
#define EXTENDED_EXPOSURE 6
#define AUDIO_TRIGGER 7
#define ACCELEROMETER 8
#define PHOTO_TRIGGER 9
#define TAKING_SHOT 10

// timer sampling constants (in ms)
#define updateLCDRate 200
#define getKeyPressedRate 30

// variables for menu state machine
int currPage;
unsigned char keyPressed;
int currChoice;

//flag to update LCD
char refreshLCD;

// vars for temp settings vals
unsigned int flashTime;
unsigned int numShots;
unsigned int exposureTime;
unsigned char audioThresh;
unsigned char accelThresh;
unsigned char photoThresh;

//vars for updating analog threshold / progres
unsigned char audioMax;
unsigned char accelMax;
unsigned char photoMin;
unsigned char progress;

// keypad states
#define KEY_RELEASE 0x00
#define KEY_DEBOUNCE_DOWN 0x01
#define KEY_PRESSED 0x02
#define KEY_DEBOUNCE_UP 0x03

//keypad vars
unsigned char keymode;
unsigned char keymaybe;
unsigned char keysymbol;

//camera init vars
#define DDRCAM DDRC

// timer variables
int updateLCDTimer, getKeyPressedTimer;

char isDigit(char key)
{
   /*
    * Helper method to tell if key pressed was a digit
    */
   if (key < 10) return 1;
   else return 0;
}

void updateMenu(void)
{
   /*
    * This function controls the menu state machine and is called
    * when a button is pressed or when the current page is animated
    */
   switch (currPage)
   {
   case MAIN_MENU:
      //do nothing if PREV is pressed
      if (keyPressed == SELECT)
      {
         if (currChoice == 0)
         {
            //first choice is setup
            currPage = SETUP;
            clearScreen();
            drawSetup();
            drawChoice(currChoice);
         }
         else if (currChoice == 1)
         {
            //reset currChoice before going to next menu
            currChoice = 0;
            currPage = START;
            //draw the start screen
            clearScreen();
            drawStart();
            drawChoice(currChoice);
         }
         else if (currChoice == 2)
         {
            //reset currChoice before going to next menu
            currChoice = 0;
            currPage = ABOUT;
            //draw the about screen
            clearScreen();
            drawAbout();
         }
      }
      else if (keyPressed == UP)
      {
         clearChoice(currChoice);
         // change currChoice, if at the top or bottom, cycle around 
         if (currChoice == 0)
            currChoice = 2;
         else currChoice--;
         // draw the updated choice
         drawChoice(currChoice);
         
      }
      else if (keyPressed == DOWN)
      {
         clearChoice(currChoice);
         // change currChoice, if at the top or bottom, cycle around 
         if (currChoice == 2)
            currChoice = 0;
         else currChoice++;
         // draw the updated choice
         drawChoice(currChoice);
      }
      break;
   case SETUP:
      if (keyPressed == PREV)
      {
         currChoice = 0;
         currPage = MAIN_MENU;
         //draw the main screen
         clearScreen();
         drawMain();
         drawChoice(currChoice);
      }
      else if (keyPressed == SELECT)
      {
         if (currChoice == 0)
         {
            //first choice is setup
            currPage = GEN_SETTINGS;
            //draw the general settings screen
         	clearScreen();
         	drawGenSettings();
         	drawGenSettingsChoice(currChoice,0);
         	//draw the saved settings
         	changeFlashTime(getFlashTime());
         	changeNumShots(getNumShots());
         	
         }
         else if (currChoice == 1)
         {
            //reset currChoice before going to next menu
            currChoice = 0;
            currPage = SHOT_TYPE;
            //draw the shot type screen
         	clearScreen();
         	drawShotType();
         	drawChoice(currChoice);
         }
      }
      else if (keyPressed == UP)
      {
         clearChoice(currChoice);
         // change currChoice, if at the top or bottom, cycle around 
         if (currChoice == 0)
            currChoice = 1;
         else currChoice--;
         // draw the updated choice
         drawChoice(currChoice);
      }
      else if (keyPressed == DOWN)
      {
         clearChoice(currChoice);
         // change currChoice, if at the top or bottom, cycle around 
         if (currChoice == 1)
            currChoice = 0;
         else currChoice++;
         // draw the updated choice
         drawChoice(currChoice);
      }
      break;
   case START:
      if (keyPressed == PREV)
      {
         //cancel shot
         currPage = MAIN_MENU;
         currChoice = 1;
         //draw the main screen
         clearScreen();
         drawMain();
         drawChoice(currChoice);
      }
      else if (keyPressed == SELECT)
      {
         if (currChoice == 0)
         {
            //first choice is extended exposure
            clearScreen();
            drawTakingShot();
            takeExtendedExposure();
            currPage = TAKING_SHOT;
         }
         if (currChoice == 1)
         {
            //second choice is audio triggered
            clearScreen();
            drawTakingShot();
            takeAudioTriggered();
            currPage = TAKING_SHOT;
         }
         if (currChoice == 2)
         {
            //third choice is accelerometer triggered
            clearScreen();
            drawTakingShot();
            takeAccelerometerTriggered();
            currPage = TAKING_SHOT;
         }
         if (currChoice == 3)
         {
            //fourth choice is photo triggered
            clearScreen();
            drawTakingShot();
            takePhotoTriggered();
            currPage = TAKING_SHOT;
         }
      }
      else if (keyPressed == UP)
      {
         clearChoice(currChoice);
         // change currChoice, if at the top or bottom, cycle around 
         if (currChoice == 0)
            currChoice = 3;
         else currChoice--;
         // draw the updated choice
         drawChoice(currChoice);
      }
      else if (keyPressed == DOWN)
      {
         clearChoice(currChoice);
         // change currChoice, if at the top or bottom, cycle around 
         if (currChoice == 3)
            currChoice = 0;
         else currChoice++;
         // draw the updated choice
         drawChoice(currChoice);
      }
      break;
   case TAKING_SHOT:
      if ((keyPressed == PREV) || (keyPressed == SELECT))
      {
         //only stop/OK button on the page
         //cancel shot
         currPage = START;
         currChoice = 0;
         //draw the main screen
         clearScreen();
         drawStart();
         drawChoice(currChoice);
         //reset shot state vars
         progress = 0;
         stopShot();
      }
      else
      {
         // if we're on the taking shot page, wait for the shot to finish
         // display a complete message when shot has been triggered
         drawProgress(progress,1);  //erase old progress
         
         if (getShotDone() == 1)
         {
            drawDoneTakingShot();
         }
         else 
         {
            progress++; 
            if (progress == 20) progress = 0;
            drawProgress(progress,0);
         }
      }
      break;
   case GEN_SETTINGS:
      if (keyPressed == PREV)
      {
         currChoice = 0;
         currPage = SETUP;
         //draw the setup screen
         clearScreen();
         drawSetup();
         drawChoice(currChoice);
      }
      else if (keyPressed == UP)
      {
         drawGenSettingsChoice(currChoice,1);
         // change currChoice, if at the top or bottom, cycle around 
         if (currChoice == 0)
            currChoice = 2;
         else currChoice--;
         // clear old and draw the updated choice
         drawGenSettingsChoice(currChoice,0);
      }
      else if (keyPressed == DOWN)
      {
         drawGenSettingsChoice(currChoice,1);
         // change currChoice, if at the top or bottom, cycle around 
         if (currChoice == 2)
            currChoice = 0;
         else currChoice++;
         // draw the updated choice
         drawGenSettingsChoice(currChoice,0);
      }
      else if ((isDigit(keyPressed) == 1) && (currChoice == 0))
      {
         //update the flash time
         //map digits 0-9 to keyPressed vals of 0-9
         flashTime = flashTime * 10 + keyPressed; 
         //check range -- only 2 digits
         if (flashTime > 9999)
            flashTime = flashTime % 10000;
	
      	changeFlashTime(flashTime);
      }
      else if ((isDigit(keyPressed) == 1) && (currChoice == 1))
      {
      	//update the num shots
      	//map digits 0-9 to keyPressed vals of 0-9
      	numShots = numShots * 10 + keyPressed;	
      	//check range -- only 2 digits
         if (numShots > 99)
            numShots = numShots % 100;
      	changeNumShots(numShots);
      }
      else if ((keyPressed == SELECT) && (currChoice == 2))
      {
      	//save these settings
      	setGenSettings(flashTime, numShots);
      }
      break;
   case SHOT_TYPE:
      if (keyPressed == PREV)
      {
         currChoice = 1;   //selection on SETUP page should be for SHOT_TYPE
         currPage = SETUP;
         //draw the setup screen
         clearScreen();
         drawSetup();
         drawChoice(currChoice);
      }
      if (keyPressed == SELECT)
      {
         if (currChoice == 0)
         {
            //first choice is extended exposure
            currPage = EXTENDED_EXPOSURE;
            //draw the extended exposure screen
         	clearScreen();
         	drawExtendedExposure();
         	drawExposureTimeChoice(0,0);
         	changeExposureTime(getExposureTime());
         }
         else if (currChoice == 1)
         {
            //reset currChoice before going to next menu
            currChoice = 0;
            currPage = AUDIO_TRIGGER;
            //draw the setup screen
         	clearScreen();
         	drawAudioTrigger();
         	audioThresh = getAudioThresh();
         	drawAnalogMeterChoice(0,audioThresh,0);
         }
         else if (currChoice == 2)
         {
            //reset currChoice before going to next menu
            currChoice = 0;
            currPage = ACCELEROMETER;
            //draw the setup screen
         	clearScreen();
         	drawAccelerometer();
         	accelThresh = getAccelThresh();
         	drawAnalogMeterChoice(0,accelThresh,0);
         }
         else if (currChoice == 3)
         {
            //reset currChoice before going to next menu
            currChoice = 0;
            currPage = PHOTO_TRIGGER;
            //draw the setup screen
         	clearScreen();
         	drawPhotoTrigger();
         	photoThresh = getPhotoThresh();
         	drawAnalogMeterChoice(0,photoThresh,0);
         }
      }
      else if (keyPressed == UP)
      {
         clearChoice(currChoice);
         // change currChoice, if at the top or bottom, cycle around 
         if (currChoice == 0)
            currChoice = 3;
         else currChoice--;
         // draw the updated choice
         drawChoice(currChoice);
      }
      else if (keyPressed == DOWN)
      {
         clearChoice(currChoice);
         // change currChoice, if at the top or bottom, cycle around 
         if (currChoice == 3)
            currChoice = 0;
         else currChoice++;
         // draw the updated choice
         drawChoice(currChoice);
      }
      break;
   case EXTENDED_EXPOSURE:
      if (keyPressed == PREV)
      {
         currPage = SHOT_TYPE;
         currChoice = 0;
         //draw the shot type screen
     	   clearScreen();
     	   drawShotType();
     	   drawChoice(currChoice);
      }
      else if (keyPressed == UP)
      {
         drawExposureTimeChoice(currChoice,1);
         // change currChoice, if at the top or bottom, cycle around 
         if (currChoice == 0)
            currChoice = 1;
         else currChoice--;
         // clear old and draw the updated choice
         drawExposureTimeChoice(currChoice,0);
      }
      else if (keyPressed == DOWN)
      {
         drawExposureTimeChoice(currChoice,1);
         // change currChoice, if at the top or bottom, cycle around 
         if (currChoice == 1)
            currChoice = 0;
         else currChoice++;
         // draw the updated choice
         drawExposureTimeChoice(currChoice,0);
      }
      else if ((isDigit(keyPressed) == 1) && (currChoice == 0))
      {
      	//update the exposure time
         //if exposureTime is -1 from programming, set to keyPressed
      	exposureTime = exposureTime * 10 + keyPressed;	
      	//check range -- only 4 digits
      	if (exposureTime > 9999)
      	   exposureTime = exposureTime % 10000;
      	changeExposureTime(exposureTime);
      }
      else if ((keyPressed == SELECT) && (currChoice == 1))
      {
      	//save these settings
      	setExtendedExposureSettings(exposureTime);
      }
      break;
   case AUDIO_TRIGGER:
      updateAnalogMeter(audioMax);
      if (keyPressed == PREV)
      {
         currPage = SHOT_TYPE;
         currChoice = 1;
         //draw the shot type screen
     	   clearScreen();
     	   drawShotType();
     	   drawChoice(currChoice);
      }
      else if (keyPressed == LEFT)
      {
         drawAnalogMeterChoice(currChoice,audioThresh,1);
         // change currChoice, if at the top or bottom, cycle around 
         if (currChoice == 0)
            currChoice = 1;
         else currChoice--;
         // clear old and draw the updated choice
         drawAnalogMeterChoice(currChoice,audioThresh,0);
      }
      else if (keyPressed == RIGHT)
      {
         drawAnalogMeterChoice(currChoice,audioThresh,1);
         // change currChoice, if at the top or bottom, cycle around 
         if (currChoice == 1)
            currChoice = 0;
         else currChoice++;
         // draw the updated choice
         drawAnalogMeterChoice(currChoice,audioThresh,0);
      }
      else if ((keyPressed == UP) && (currChoice == 0))
      {
      	//clear the old threshold marker
      	drawThresholdMarker(audioThresh, 2);
      	//move the threshold marker up
      	if (audioThresh <= 245) audioThresh+=10;
      	//draw new threshold marker
      	drawThresholdMarker(audioThresh, 0);
      }
      else if ((keyPressed == DOWN) && (currChoice == 0))
      {
      	//clear the old threshold marker
      	drawThresholdMarker(audioThresh, 2);
      	//move the threshold marker up
      	if (audioThresh >= 10) audioThresh-=10;
      	//draw new threshold marker
      	drawThresholdMarker(audioThresh, 0);
      }
      else if ((keyPressed == SELECT) && (currChoice == 1))
      {
      	//save these settings
      	setAudioTriggerSettings(audioThresh);
      }
      break;
   case ACCELEROMETER:
      updateAnalogMeter(accelMax);
      if (keyPressed == PREV)
      {
         currPage = SHOT_TYPE;
         currChoice = 2;
         //draw the shot type screen
     	   clearScreen();
     	   drawShotType();
     	   drawChoice(currChoice);
      }
      else if (keyPressed == LEFT)
      {
         drawAnalogMeterChoice(currChoice,accelThresh,1);
         // change currChoice, if at the top or bottom, cycle around 
         if (currChoice == 0)
            currChoice = 1;
         else currChoice--;
         // clear old and draw the updated choice
         drawAnalogMeterChoice(currChoice,accelThresh,0);
      }
      else if (keyPressed == RIGHT)
      {
         drawAnalogMeterChoice(currChoice,accelThresh,1);
         // change currChoice, if at the top or bottom, cycle around 
         if (currChoice == 1)
            currChoice = 0;
         else currChoice++;
         // draw the updated choice
         drawAnalogMeterChoice(currChoice,accelThresh,0);
      }
      else if ((keyPressed == UP) && (currChoice == 0))
      {
      	//clear the old threshold marker
      	drawThresholdMarker(accelThresh, 2);
      	//move the threshold marker up
      	if (accelThresh <= 245) accelThresh+=10;
      	//draw new threshold marker
      	drawThresholdMarker(accelThresh, 0);
      }
      else if ((keyPressed == DOWN) && (currChoice == 0))
      {
      	//clear the old threshold marker
      	drawThresholdMarker(accelThresh, 2);
      	//move the threshold marker up
      	if (accelThresh >= 10) accelThresh-=10;
      	//draw new threshold marker
      	drawThresholdMarker(accelThresh, 0);
      }
      else if ((keyPressed == SELECT) && (currChoice == 1))
      {
      	//save these settings
      	setAccelerometerSettings(accelThresh);
      }
      break;
   case PHOTO_TRIGGER:
      updateAnalogMeter(photoMin);
      if (keyPressed == PREV)
      {
         currPage = SHOT_TYPE;
         currChoice = 3;
         //draw the shot type screen
     	   clearScreen();
     	   drawShotType();
     	   drawChoice(currChoice);
      }
      else if (keyPressed == LEFT)
      {
         drawAnalogMeterChoice(currChoice,photoThresh,1);
         // change currChoice, if at the top or bottom, cycle around 
         if (currChoice == 0)
            currChoice = 1;
         else currChoice--;
         // clear old and draw the updated choice
         drawAnalogMeterChoice(currChoice,photoThresh,0);
      }
      else if (keyPressed == RIGHT)
      {
         drawAnalogMeterChoice(currChoice,photoThresh,1);
         // change currChoice, if at the top or bottom, cycle around 
         if (currChoice == 1)
            currChoice = 0;
         else currChoice++;
         // draw the updated choice
         drawAnalogMeterChoice(currChoice,photoThresh,0);
      }
      else if ((keyPressed == UP) && (currChoice == 0))
      {
      	//clear the old threshold marker
      	drawThresholdMarker(photoThresh, 2);
      	//move the threshold marker up
      	if (photoThresh < 245) photoThresh+=10;
      	//draw new threshold marker
      	drawThresholdMarker(photoThresh, 0);
      }
      else if ((keyPressed == DOWN) && (currChoice == 0))
      {
      	//clear the old threshold marker
      	drawThresholdMarker(photoThresh, 2);
      	//move the threshold marker up
      	if (photoThresh > 10) photoThresh-=10;
      	//draw new threshold marker
      	drawThresholdMarker(photoThresh, 0);
      }
      else if ((keyPressed == SELECT) && (currChoice == 1))
      {
      	//save these settings
      	setPhotoTriggerSettings(photoThresh);
      }
      break;
   case ABOUT:
      // display a message about the device and how to use it
      if (keyPressed == PREV)
      {
         //cancel shot
         currPage = MAIN_MENU;
         currChoice = 2;
         //draw the shot type screen
     	   clearScreen();
     	   drawMain();
     	   drawChoice(currChoice);
      }
      break;     
   }
}

unsigned char keypadScan(void)
{
   /*
    * This method scans the keypad in the same fashion as done in lab2. It 
    * first powers the lower 4 bits and reads the higher, then does the 
    * opposite to read the lower 4 bits.
    * 
    * Button hookup:
    * 0 -- UP GRAY
    * 1 -- SELECT GRAY
    * 2 -- RIGHT GRAY
    * 3 -- DOWN GRAY
    * 4 -- SEVEN RED
    * 5 -- ZERO RED
    * 6 -- POWER RED
    * 7 -- DOWN RED
    */
   
   unsigned char key;
   
   //get lower nibble
   DDRD = 0x0f;
   PORTD = 0xf0;
   _delay_us(5);
   key = PIND;

   //get upper nibble
   DDRD = 0xf0;
   PORTD = 0x0f;
   _delay_us(5);
   key = key | PIND;

   return key;
}

unsigned char buttonLookup(unsigned char key)
{
   if (key == 0xee) return LEFT;
   else if (key == 0xed) return 0X01;
   else if (key == 0xeb) return 0X04;
   else if (key == 0xe7) return 0X07;
   else if (key == 0xde) return 0X02;
   else if (key == 0xdd) return 0X05;
   else if (key == 0xdb) return 0X08;
   else if (key == 0xd7) return 0X00;
   else if (key == 0xbe) return 0X03;
   else if (key == 0xbd) return 0X06;
   else if (key == 0xbb) return 0X09;
   else if (key == 0xb7) return PREV;
//   else if (key == 0x7e) return UP;
   //since the LCD displays upside down
   //hack and make UP button send DOWN signal
   //and vice versa
   else if (key == 0x7e) return DOWN;
   else if (key == 0x7d) return SELECT;
   else if (key == 0x7b) return RIGHT;
   //else if (key == 0x77) return DOWN;
   else if (key == 0x77) return UP;
   else return NONE; 
}

void getKeyPress(void)
{
    unsigned char keycode;

    // scan keypad for keycode
    keycode = keypadScan();

    if (keymode == KEY_RELEASE)
    {
        // check for keycode
        if (keycode != 0x00)
        {
            keymaybe = keycode;
            keymode = KEY_DEBOUNCE_DOWN;
        }
    }
    else if (keymode == KEY_DEBOUNCE_DOWN)
    {
        // check for keycode match
        if (keycode == keymaybe)
        {
            keymode = KEY_PRESSED;
        }
        else
        {
            keymode = KEY_RELEASE;
        }
    }
    else if (keymode == KEY_PRESSED)
    {
		//lookup keysymbol based on pressed key
		keysymbol = buttonLookup(keymaybe);
        // check for keycode mismatch
        if (keycode != keymaybe)
        {
            keymode = KEY_DEBOUNCE_UP;
        }
    }
    else if (keymode == KEY_DEBOUNCE_UP)
    {
        // check for keycode match
        if (keycode == keymaybe)
        {
            keymode = KEY_PRESSED;
        }
        else
        {
            // release for next button
            keymode = KEY_RELEASE;
            //declare key pressed here, update the menu
            if (keysymbol != NONE)
			   {
               keyPressed = keysymbol;
               refreshLCD = 1;
            }
        }
    }
}

// timer 2 compare ISR, 100us timer
ISR(TIMER2_COMP_vect) {
  if (shotType == EXT_EXP) {
	if (eeTimer--==0) {
	   if (shotStateTimer > 0) shotStateTimer--; // Decrease this every 1 second instead of every 100us.
	  eeTimer = 10000; // Reset to 1 second timer
	}
  } else {
	if (shotStateTimer > 0) shotStateTimer--;
  }
}

//timer 0 compare ISR, 1ms timer
ISR(TIMER0_COMP_vect)
{      
   // decrement update all timeres
   if (updateLCDTimer > 0) updateLCDTimer--;
   if (getKeyPressedTimer > 0) getKeyPressedTimer--;
   
   //shot state timer from camera.c
   //if (shotStateTimer > 0) shotStateTimer--;
}  


void initialize(void)
{  
   //set up timer 0 for 1 mSec timebase 
   TIMSK = 0x82;      //turn on timer 0 cmp match ISR 
   OCR0 = 250;     //set the compare re to 250 time ticks
   OCR2 = 25;      // set the compare to 25 time ticks (100us).
   TCCR0=0x0b; // clk/64 prescalar and clear-on-match
   TCCR2=0x0c; // clk/64 prescalar and clear-on-match (tccr0 and tccr2 DO NOT behave the same... see atmega32 datasheet)
   updateLCDTimer = updateLCDRate;
   getKeyPressedTimer = getKeyPressedRate;
   
   //start up the lcd
   lcdInit();
   clearScreen();
   drawMain();
   drawChoice(0);
   refreshLCD = 0;
   
   //init state machine vars
   currPage = MAIN_MENU;
   currChoice = 0;
   keyPressed = NONE;
   progress = 0;
   
   //start up keypad state machine
   keymode = KEY_RELEASE;
   keymaybe = 0x00;
   keysymbol = NONE;
   
   //set up ADC for camera sensors
   //set up the ADC to use AVCC with external capacitor at AREF
   //internal 2.56V ref use ADMUX = 0xe0
   ADMUX = 0x60;
   //set ADC enabled, start conversion, prescalar to 128 
   ADCSRA = 0xc7;
   
   //set flash and shutter directions
   //start flash and shutter off
   sbi(DDRCAM, SHUTTER_PIN);
   sbi(DDRCAM, FLASH_PIN);
   flash(0);
   shutter(0);
   
   //read in initial thresholds from eeprom
   audioThresh = getAudioThresh();
   accelThresh = getAccelThresh();
   photoThresh = getPhotoThresh();
   
   //enable global interrupts
   sei();

}

char onDynamicPage(void)
{
   /* 
    * This method returns true if currPage needs to be updated every
    * updateLCD cycle.
    */
   char dynamic = 0;
   if ((currPage == TAKING_SHOT) ||
         (currPage == AUDIO_TRIGGER) ||
         (currPage == ACCELEROMETER) ||
         (currPage == PHOTO_TRIGGER))
      dynamic = 1;
   return dynamic;
}

void updateAnalogMaxes()
{
   /*
    * Update the max value every cycle and change it
    */
   unsigned char currVal = 0;
   if (currPage == AUDIO_TRIGGER) {
      currVal = getAudioVal();
      if(currVal > audioMax) audioMax = currVal;
   }
   if (currPage == ACCELEROMETER) {
      currVal = getAccelerometerVal();
      if(currVal > accelMax) accelMax = currVal;
   }
   if (currPage == PHOTO_TRIGGER) {
      currVal = getPhotoVal();
      if(currVal < photoMin) photoMin = currVal;
   }
}

int main(void)
{
  initialize();
  
  //main task scheduler loop 
  while(1) 
  { 
	
	if (updateLCDTimer == 0)
    {
	    updateLCDTimer = updateLCDRate;
       if (refreshLCD == 1)
       {
          updateMenu();
          refreshLCD = 0;
          keyPressed = NONE;
       }
       if (onDynamicPage() == 1)
       {
          updateMenu();
       }
       // clear analog maxes
       audioMax = 0;
       accelMax = 0;
       photoMin = 255;
	}
    if(getKeyPressedTimer == 0)
    {
	    getKeyPressedTimer = getKeyPressedRate;
       getKeyPress();
	}
	//check shot on every iteration -- tightest loop possible
	checkShot();
	updateAnalogMaxes();
  }
}
